#!/usr/bin/env python
# encoding: utf-8

# The MIT License (MIT)

# Copyright (c) 2012-2014 CNRS (Hervé BREDIN - http://herve.niderb.fr)

# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:

# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.

# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

from __future__ import unicode_literals

import numpy as np

from ..matcher import LabelMatcherWithUnknownSupport
from pyannote.core import Annotation, Unknown
from pyannote.core.matrix import LabelMatrix
from pyannote.algorithms.tagging import DirectTagger

from ..matcher import MATCH_CORRECT, MATCH_CONFUSION, \
    MATCH_MISSED_DETECTION, MATCH_FALSE_ALARM

from ..identification import UEMSupportMixin

REFERENCE_TOTAL = 'reference'
HYPOTHESIS_TOTAL = 'hypothesis'


class IdentificationErrorAnalysis(UEMSupportMixin, object):
    """

    Parameters
    ----------
    matcher : `Matcher`, optional
        Defaults to `LabelMatcherWithUnknownSupport` instance
        i.e. two Unknowns are always considered as correct.
    unknown : bool, optional
        Set `unknown` to True (default) to take `Unknown` instances into
        account. Set it to False to get rid of them before evaluation.
    merge_unknowns : bool, optional
        See all `Unknown` instances as one unique label. Defaults to False.
    collar : float, optional
        Duration (in seconds) of collars removed from evaluation around
        boundaries of reference segments.

    """

    def __init__(self, matcher=None, unknown=True, merge_unknowns=False,
                 collar=0.):

        super(IdentificationErrorAnalysis, self).__init__()

        if matcher is None:
            matcher = LabelMatcherWithUnknownSupport()
        self.matcher = matcher
        self.unknown = unknown
        self.merge_unknowns = merge_unknowns
        self.collar = collar

        self._tagger = DirectTagger()

    def _merge_unknowns(self, reference, hypothesis):

        # create new unique `Unknown` instance label
        unknown = Unknown('?')

        # gather reference and hypothesis unknown labels
        rUnknown = [u for u in reference.labels()
                    if isinstance(u, Unknown)]
        hUnknown = [u for u in hypothesis.labels()
                    if isinstance(u, Unknown)]

        # replace them all by unique unknown label
        translation = {u: unknown for u in rUnknown + hUnknown}
        reference = reference.translate(translation)
        hypothesis = hypothesis.translate(translation)

        return reference, hypothesis

    def annotation(self, reference, hypothesis, uem=None, uemified=False):
        """Get error analysis as `Annotation`

        Labels are (status, reference_label, hypothesis_label) tuples.
        `status` is either 'correct', 'confusion', 'missed detection' or
        'false alarm'.
        `reference_label` is None in case of 'false alarm'.
        `hypothesis_label` is None in case of 'missed detection'.

        Parameters
        ----------
        uemified : bool, optional
            Returns "uemified" version of reference and hypothesis.
            Defaults to False.

        Returns
        -------
        errors : `Annotation`

        """

        reference, hypothesis = self.uemify(
            reference, hypothesis, uem=uem, collar=self.collar)

        # merge Unknown instance labels into one unique `Unknown` instance
        if self.unknown and self.merge_unknowns:
            reference, hypothesis = self._merge_unknowns(reference, hypothesis)

        # common (up-sampled) timeline
        common_timeline = reference.get_timeline().union(
            hypothesis.get_timeline())
        common_timeline = common_timeline.segmentation()

        # align reference on common timeline
        R = self._tagger(reference, common_timeline)

        # translate and align hypothesis on common timeline
        H = self._tagger(hypothesis, common_timeline)

        errors = Annotation(uri=reference.uri, modality=reference.modality)

        # loop on all segments
        for segment in common_timeline:

            # list of labels in reference segment
            rlabels = R.get_labels(segment, unknown=self.unknown, unique=False)

            # list of labels in hypothesis segment
            hlabels = H.get_labels(segment, unknown=self.unknown, unique=False)

            _, details = self.matcher(rlabels, hlabels)

            for r, h in details[MATCH_CORRECT]:
                track = errors.new_track(segment, prefix=MATCH_CORRECT)
                errors[segment, track] = (MATCH_CORRECT, r, h)

            for r, h in details[MATCH_CONFUSION]:
                track = errors.new_track(segment, prefix=MATCH_CONFUSION)
                errors[segment, track] = (MATCH_CONFUSION, r, h)

            for r in details[MATCH_MISSED_DETECTION]:
                track = errors.new_track(segment,
                                         prefix=MATCH_MISSED_DETECTION)
                errors[segment, track] = (MATCH_MISSED_DETECTION, r, None)

            for h in details[MATCH_FALSE_ALARM]:
                track = errors.new_track(segment, prefix=MATCH_FALSE_ALARM)
                errors[segment, track] = (MATCH_FALSE_ALARM, None, h)

        if uemified:
            return reference, hypothesis, errors
        else:
            return errors

    def matrix(self, reference, hypothesis, uem=None):

        reference, hypothesis, errors = self.annotation(
            reference, hypothesis, uem=uem, uemified=True)

        chart = errors.chart()

        # rLabels contains reference labels
        # hLabels contains hypothesis labels confused with a reference label
        # falseAlarmLabels contains false alarm hypothesis labels that do not
        # exist in reference labels // corner case  //
        rLabels, hLabels, falseAlarmLabels = set([]), set([]), set([])
        for (status, rLabel, hLabel), _ in chart:

            # labels for confusion matrix
            if status in {MATCH_CORRECT, MATCH_CONFUSION}:
                rLabels.add(rLabel)
                hLabels.add(hLabel)

            # missed reference label
            if status == MATCH_MISSED_DETECTION:
                rLabels.add(rLabel)

            # false alarm hypothesis labels
            if status == MATCH_FALSE_ALARM:
                falseAlarmLabels.add(hLabel)

        # make sure only labels that do not exist in reference are ketp
        falseAlarmLabels = falseAlarmLabels - rLabels

        # sort these sets of labels
        cmp_func = reference._cmp_labels
        falseAlarmLabels = sorted(falseAlarmLabels, cmp=cmp_func)
        rLabels = sorted(rLabels, cmp=cmp_func)
        hLabels = sorted(hLabels, cmp=cmp_func)

        # append false alarm labels as last 'reference' labels
        # (make sure to mark them as such)
        rLabels = rLabels + [(MATCH_FALSE_ALARM, hLabel)
                             for hLabel in falseAlarmLabels]

        # prepend duration columns before the detailed confusion matrix
        hLabels = [
            REFERENCE_TOTAL, HYPOTHESIS_TOTAL,
            MATCH_CORRECT, MATCH_CONFUSION,
            MATCH_FALSE_ALARM, MATCH_MISSED_DETECTION
        ] + hLabels

        # initialize empty matrix
        matrix = LabelMatrix(
            rows=rLabels, columns=hLabels,
            data=np.zeros((len(rLabels), len(hLabels)))
        )

        # loop on chart
        for (status, rLabel, hLabel), duration in chart:

            # increment confusion matrix
            if status in {MATCH_CORRECT, MATCH_CONFUSION}:
                matrix[rLabel, hLabel] += duration

            # corner case for ('false alarm', None, hLabel)
            if status == MATCH_FALSE_ALARM:
                # hLabel is also a reference label
                if hLabel in rLabels:
                    rLabel = hLabel
                # hLabel comes out of nowhere (ie. is not even in reference)
                else:
                    rLabel = (MATCH_FALSE_ALARM, hLabel)

            # increment status column
            matrix[rLabel, status] += duration

        # total reference and hypothesis duration
        for rLabel in rLabels:

            if isinstance(rLabel, tuple) and rLabel[0] == MATCH_FALSE_ALARM:
                r = 0.
                h = hypothesis.label_duration(rLabel[0])
            else:
                r = reference.label_duration(rLabel)
                h = hypothesis.label_duration(rLabel)

            matrix[rLabel, REFERENCE_TOTAL] = r
            matrix[rLabel, HYPOTHESIS_TOTAL] = h

        return matrix



#!/usr/bin/env python
# encoding: utf-8

# Copyright 2012-2014 CNRS (Herve BREDIN -- bredin@limsi.fr)

# This file is part of PyAnnote.
#
#     PyAnnote is free software: you can redistribute it and/or modify
#     it under the terms of the GNU General Public License as published by
#     the Free Software Foundation, either version 3 of the License, or
#     (at your option) any later version.
#
#     PyAnnote is distributed in the hope that it will be useful,
#     but WITHOUT ANY WARRANTY; without even the implied warranty of
#     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#     GNU General Public License for more details.
#
#     You should have received a copy of the GNU General Public License
#     along with PyAnnote.  If not, see <http://www.gnu.org/licenses/>.


from pyannote.base.annotation import Annotation
from pyannote.base.timeline import Timeline

class SegmentationError(object):

    def __init__(self):
        super(SegmentationError, self).__init__()
    
    def __call__(self, reference, hypothesis):

        if isinstance(reference, Annotation):
            reference = reference.get_timeline()

        if isinstance(hypothesis, Annotation):
            hypothesis = hypothesis.get_timeline()

        # over-segmentation
        over = Timeline(uri=reference.uri)
        prev_r = reference[0]
        intersection = []
        for r, h in reference.co_iter(hypothesis):

            if r != prev_r:
                intersection = sorted(intersection)
                for _, segment in intersection[:-1]:
                    over.add(segment)
                intersection = []
                prev_r = r

            segment = r & h
            intersection.append((segment.duration, segment)) 

        intersection = sorted(intersection)
        for _, segment in intersection[:-1]:
            over.add(segment)

        # under-segmentation
        under = Timeline(uri=reference.uri)
        prev_h = hypothesis[0]
        intersection = []
        for h, r in hypothesis.co_iter(reference):

            if h != prev_h:
                intersection = sorted(intersection)
                for _, segment in intersection[:-1]:
                    under.add(segment)
                intersection = []
                prev_h = h

            segment = h & r
            intersection.append((segment.duration, segment)) 

        intersection = sorted(intersection)
        for _, segment in intersection[:-1]:
            under.add(segment)

        # extent
        extent = reference.extent()

        # correct (neither under- nor over-segmented)
        correct = under.union(over).gaps(focus=extent)

        # frontier error (both under- and over-segmented)
        frontier = under.crop(over)

        # under-segmented
        not_over = over.gaps(focus=extent)
        only_under = under.crop(not_over)

        # over-segmented
        not_under = under.gaps(focus=extent)
        only_over = over.crop(not_under)

        status = Annotation(uri=reference.uri)
        for segment in correct:
            status[segment, '_'] = 'correct'
        for segment in frontier:
            status[segment, '_'] = 'frontier'
        for segment in only_over:
            status[segment, '_'] = 'over'
        for segment in only_under:
            status[segment, '_'] = 'under'

        return status.smooth()





#!/usr/bin/env python
# encoding: utf-8

# The MIT License (MIT)

# Copyright (c) 2012-2014 CNRS (Hervé BREDIN - http://herve.niderb.fr)

# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:

# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.

# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

from __future__ import unicode_literals

from identification import IDMatcher
from pyannote.base.annotation import Unknown


class REPEREIDMatcher(IDMatcher):
    """
    REPERE ID matcher:
    Two IDs match if:
    * they are both anonymous, or
    * they are both named and equal.

    """

    def __init__(self):
        super(REPEREIDMatcher, self).__init__()

    def __call__(self, identifier1, identifier2):
        return (self.is_anonymous(identifier1) and
                self.is_anonymous(identifier2)) \
            or (identifier1 == identifier2 and
                self.is_named(identifier1) and
                self.is_named(identifier2))

    def ncorrect(self, identifiers1, identifiers2):
        # Slightly faster than inherited .ncorrect() method
        named1 = self.named_from(identifiers1)
        named2 = self.named_from(identifiers2)
        anonymous1 = self.anonymous_from(identifiers1)
        anonymous2 = self.anonymous_from(identifiers2)
        return len(named1 & named2) + min(len(anonymous1), len(anonymous2))

    # ------------------------------------------------------------ #
    def is_anonymous(self, identifier):
        return isinstance(identifier, Unknown) or \
            (isinstance(identifier, str) and
                (identifier[:8] == 'Inconnu_' or
                 identifier[:7] == 'speaker'))

    def anonymous_from(self, identifiers):
        return set([identifier for identifier in identifiers
                    if self.is_anonymous(identifier)])
    # ------------------------------------------------------------ #

    def is_named(self, identifier):
        return not self.is_anonymous(identifier)

    def named_from(self, identifiers):
        return set([identifier for identifier in identifiers
                    if self.is_named(identifier)])

# --------------------------------------------------------------------------- #

from base import BaseMetric
from pyannote.algorithm.tagging.segment import ArgMaxDirectTagger

EGER_TOTAL = 'total'

EGER_REF_NAME = 'total named in reference'
EGER_REF_ANON = 'total anonymous in reference'
EGER_HYP_NAME = 'total named in hypothesis'
EGER_HYP_ANON = 'total anonymous in hypothesis'

EGER_CORRECT_NAME = 'correct named'
EGER_CORRECT_ANON = 'correct anonymous'

EGER_CONFUSION_NAME_NAME = 'confusion named/named'
EGER_CONFUSION_NAME_ANON = 'confusion named/anonymous'
EGER_CONFUSION_ANON_NAME = 'confusion anonymous/named'

EGER_FALSE_ALARM_NAME = 'false alarm named'
EGER_FALSE_ALARM_ANON = 'false alarm anonymous'

EGER_MISS_NAME = 'miss named'
EGER_MISS_ANON = 'miss anonymous'

EGER_NAME = 'estimated global error rate'


class EstimatedGlobalErrorRate(BaseMetric):
    """
    Estimated Global Error Rate

    Pour chaque image annotée (annotated timeline) de la reference, la liste
    des personnes présentes et/ou parlant à l’instant associé est constituée,
    et ce du point de vue référence et du point de vue système.

    Ces deux listes sont comparées en associant les personnes une à une, chaque
    personne ne pouvant être associée au plus qu’une fois. Une association
    entre deux personnes nommées compte pour un correct, tout comme
    l’association entre deux anonymes. L’association entre deux personnes avec
    des noms différents ou entre un nommé et un anonyme donne une confusion.
    Chaque personne de l’hypothèse non associée compte pour une fausse alarme,
    et chaque personne de la référence non associée pour un oubli. Un coût est
    associé par confusion, et un par oubli/fausse alarme.

    De toutes les associations possibles est choisie celle qui donne le coût
    total (erreur pour l’image) le plus faible.
    La somme de tous ces comptes d’erreur par image permet d’obtenir le nombre
    d’erreurs global. Le nombre global d’entrées attendues est lui aussi
    comptabilisé en cumulant le nombre de personnes présentes dans la référence
    à chaque image. Le taux d’erreur est alors le nombre d’erreurs global
    divisé par le nombre global d’entrées attendues.

    Nous nous proposons d’utiliser un coût de 1 pour oubli/fausse alarme et de
    0,5 pour confusion.

    Example

    >>> xgtf = XGTFParser(path2xgtf, path2idx)
    >>> reference = xgtf.head()
    >>> annotated = xgtf.annotated()
    >>> hypothesis = my_super_algorithm()
    >>> eger = EstimatedGlobalErrorRate()
    >>> error_rate = eger(reference, hypothesis, annotated=annotated)

    """

    @classmethod
    def metric_name(cls):
        return EGER_NAME

    @classmethod
    def metric_components(cls):
        return [EGER_CONFUSION_NAME_NAME, EGER_CONFUSION_NAME_ANON,
                EGER_CONFUSION_ANON_NAME,
                EGER_FALSE_ALARM_NAME, EGER_FALSE_ALARM_ANON,
                EGER_MISS_NAME, EGER_MISS_ANON,
                EGER_REF_NAME, EGER_REF_ANON, EGER_TOTAL,
                EGER_HYP_NAME, EGER_HYP_ANON,
                EGER_CORRECT_NAME, EGER_CORRECT_ANON]

    def __init__(self, confusion=1., anonymous=False):

        super(EstimatedGlobalErrorRate, self).__init__()
        self.matcher = REPEREIDMatcher()
        self.confusion = confusion
        self.anonymous = anonymous
        self.tagger = ArgMaxDirectTagger()

    def __get_precision(self):
        if self[EGER_HYP_NAME] > 0:
            precision = 1. * self[EGER_CORRECT_NAME] / self[EGER_HYP_NAME]
        else:
            precision = 1.
        return precision
    precision = property(fget=__get_precision)
    """Overall precision."""

    def __get_recall(self):
        if self[EGER_HYP_NAME] > 0:
            recall = 1. * self[EGER_CORRECT_NAME] / self[EGER_REF_NAME]
        else:
            recall = 1.
        return recall
    recall = property(fget=__get_recall)
    """Overall recall."""

    def __get_fmeasure(self):
        precision = self.precision
        recall = self.recall
        return 2 * precision * recall / (precision + recall)
    f_measure = property(fget=__get_fmeasure)
    """Overall F1-measure."""

    def _get_details(self, reference, hypothesis, annotated=None):

        if annotated is None:
            raise ValueError("'annotated' argument is mandatory.")

        detail = self._init_details()

        reference = self.tagger(reference, annotated)
        hypothesis = self.tagger(hypothesis, annotated)

        for frame in annotated:

            ref = reference.get_labels(frame)
            hyp = hypothesis.get_labels(frame)

            if not self.anonymous:
                ref = self.matcher.named_from(ref)
                hyp = self.matcher.named_from(hyp)

            name_ref = self.matcher.named_from(ref)
            name_hyp = self.matcher.named_from(hyp)
            detail[EGER_REF_NAME] += len(name_ref)
            detail[EGER_HYP_NAME] += len(name_hyp)

            anon_ref = ref - name_ref
            anon_hyp = hyp - name_hyp
            detail[EGER_REF_ANON] += len(anon_ref)
            detail[EGER_HYP_ANON] += len(anon_hyp)

            detail[EGER_TOTAL] += detail[EGER_REF_NAME] + detail[EGER_REF_ANON]

            # correct named/named matches
            detail[EGER_CORRECT_NAME] += len(name_ref & name_hyp)
            for known in name_ref & name_hyp:
                name_ref.remove(known)
                name_hyp.remove(known)

            # correct anonymous/anonymous matches
            n = min(len(anon_ref), len(anon_hyp))
            detail[EGER_CORRECT_ANON] += n
            for i in range(n):
                anon_ref.pop()
                anon_hyp.pop()

            # named/named confusion
            n = min(len(name_ref), len(name_hyp))
            detail[EGER_CONFUSION_NAME_NAME] += n
            for i in range(n):
                name_ref.pop()
                name_hyp.pop()

            # named/anonymous confusion
            n = min(len(name_ref), len(anon_hyp))
            detail[EGER_CONFUSION_NAME_ANON] += n
            for i in range(n):
                name_ref.pop()
                anon_hyp.pop()

            # anonymous/named confusion
            n = min(len(anon_ref), len(name_hyp))
            detail[EGER_CONFUSION_ANON_NAME] += n
            for i in range(n):
                anon_ref.pop()
                name_hyp.pop()

            # miss
            detail[EGER_MISS_NAME] += len(name_ref)
            detail[EGER_MISS_ANON] += len(anon_ref)

            # false alarm
            detail[EGER_FALSE_ALARM_NAME] += len(name_hyp)
            detail[EGER_FALSE_ALARM_ANON] += len(anon_hyp)

        return detail

    def _get_rate(self, detail):
        numerator = self.confusion * detail[EGER_CONFUSION_NAME_NAME] + \
            self.confusion * detail[EGER_CONFUSION_NAME_ANON] + \
            self.confusion * detail[EGER_CONFUSION_ANON_NAME] + \
            1. * detail[EGER_FALSE_ALARM_NAME] + \
            1. * detail[EGER_FALSE_ALARM_ANON] + \
            1. * detail[EGER_MISS_NAME] + \
            1. * detail[EGER_MISS_ANON]
        denominator = 1. * detail[EGER_REF_NAME] + \
            1. * detail[EGER_REF_ANON]
        if denominator == 0.:
            if numerator == 0:
                return 0.
            else:
                return 1.
        else:
            return numerator/denominator

    def _pretty(self, detail):

        string = ""

        ref_name = detail[EGER_REF_NAME]
        ref_anon = detail[EGER_REF_ANON]
        string += "  - reference entries: %d (%d named, %d anonymous)\n" % \
                  (ref_name+ref_anon, ref_name, ref_anon)

        hyp_name = detail[EGER_HYP_NAME]
        hyp_anon = detail[EGER_HYP_ANON]
        string += "  - hypothesis entries: %d (%d named, %d anonymous)\n" % \
                  (hyp_name+hyp_anon, hyp_name, hyp_anon)

        correct_name = detail[EGER_CORRECT_NAME]
        correct_anon = detail[EGER_CORRECT_ANON]
        string += "  - correct: %d (%d named, %d anonymous)\n" % \
                  (correct_name+correct_anon, correct_name, correct_anon)

        confusion_nn = detail[EGER_CONFUSION_NAME_NAME]
        confusion_na = detail[EGER_CONFUSION_NAME_ANON]
        confusion_an = detail[EGER_CONFUSION_ANON_NAME]
        string += "  - confusions: %d (%d n-n, %d n-a, %d a-n)\n" % \
                  (confusion_nn+confusion_na+confusion_an,
                   confusion_nn, confusion_na, confusion_an)

        miss_name = detail[EGER_MISS_NAME]
        miss_anon = detail[EGER_MISS_ANON]
        string += "  - miss: %d (%d named, %d anonymous)\n" % \
                  (miss_name+miss_anon, miss_name, miss_anon)

        fa_name = detail[EGER_FALSE_ALARM_NAME]
        fa_anon = detail[EGER_FALSE_ALARM_ANON]
        string += "  - fa: %d (%d named, %d anonymous)\n" % \
                  (fa_name+fa_anon, fa_name, fa_anon)

        if detail[EGER_HYP_NAME] > 0:
            precision = 1. * detail[EGER_CORRECT_NAME] / detail[EGER_HYP_NAME]
        else:
            precision = 1.
        string += "  - precision (named): %.2f %%\n" % (100*precision)

        if detail[EGER_HYP_NAME] > 0:
            recall = 1. * detail[EGER_CORRECT_NAME] / detail[EGER_REF_NAME]
        else:
            recall = 1.
        string += "  - recall (named): %.2f %%\n" % (100*recall)

        if (precision + recall > 0):
            fmeasure = 2 * precision * recall / (precision + recall)
        else:
            fmeasure = 0.
        string += "  - F1-measure (named): %.2f %%\n" % (100*fmeasure)

        string += "  - EGER: %.2f %%\n" % (100*detail[self.name])

        return string


if __name__ == "__main__":
    import doctest
    doctest.testmod()
